@@ -14,7 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  */
 
@@ -22,10 +22,10 @@
 
 #include "ev-document-find.h"
 
-EV_DEFINE_INTERFACE (EvDocumentFind, ev_document_find, 0)
+G_DEFINE_INTERFACE (EvDocumentFind, ev_document_find, 0)
 
 static void
-ev_document_find_class_init (EvDocumentFindIface *klass)
+ev_document_find_default_init (EvDocumentFindInterface *klass)
 {
 }
 
@@ -35,7 +35,7 @@ ev_document_find_find_text (EvDocumentFind *document_find,
 			    const gchar    *text,
 			    gboolean        case_sensitive)
 {
-	EvDocumentFindIface *iface = EV_DOCUMENT_FIND_GET_IFACE (document_find);
+	EvDocumentFindInterface *iface = EV_DOCUMENT_FIND_GET_IFACE (document_find);
 	
 	return iface->find_text (document_find, page, text, case_sensitive);
 }
